@@ -9,9 +9,9 @@ a simple API through which apps can find, fetch and install updates, and a
 bootstrapping mechanism that keeps the app safe in the face of failed or
 partial updates.
 
-Esky is currently capable of freezing apps with bbfreeze, cxfreeze and py2exe.
-Adding support for other freezer programs should be straightforward; patches
-will be gratefully accepted.
+Esky is currently capable of freezing apps with bbfreeze, cxfreeze py2exe and
+py2app. Adding support for other freezer programs should be straightforward;
+patches will be gratefully accepted.
 
 The main interface is the 'Esky' class, which represents a frozen app.  An Esky
 must be given the path to the top-level directory of the frozen app, and a
@@ -91,7 +91,7 @@ call the "cleanup" method on their esky.
 from __future__ import with_statement
 
 __ver_major__ = 0
-__ver_minor__ = 4
+__ver_minor__ = 5
 __ver_patch__ = 0
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
@@ -108,13 +108,17 @@ try:
     import threading
 except ImportError:
     threading = None
+try:
+    import fcntl
+except ImportError:
+    fcntl = None
             
 
 from esky.errors import *
 from esky.fstransact import FSTransaction
 from esky.finder import DefaultVersionFinder
-from esky.util import is_core_dependency, split_app_version, join_app_version,\
-                      parse_version, get_best_version
+from esky.util import split_app_version, join_app_version,\
+                      parse_version, get_best_version, appdir_from_executable
 
 
 class Esky(object):
@@ -138,10 +142,15 @@ class Esky(object):
 
     lock_timeout = 60*60  # 1 hour
 
-    def __init__(self,appdir,version_finder=None):
-        if os.path.isfile(appdir):
-            appdir = os.path.dirname(os.path.dirname(appdir))
-        self.appdir = appdir
+    def __init__(self,appdir_or_exe,version_finder=None):
+        if os.path.isfile(appdir_or_exe):
+            self.appdir = appdir_from_executable(appdir_or_exe)
+            vdir = appdir_or_exe[len(self.appdir):].split(os.sep)[1]
+            details = split_app_version(vdir)
+            self.name,self.active_version,self.platform = details
+        else:
+            self.active_version = None
+            self.appdir = appdir_or_exe
         self.reinitialize()
         self._lock_count = 0
         self.version_finder = version_finder
@@ -171,7 +180,8 @@ class Esky(object):
         best_version = get_best_version(self.appdir)
         if best_version is None:
             raise EskyBrokenError("no frozen versions found")
-        self.name,self.version,self.platform = split_app_version(best_version)
+        details = split_app_version(best_version)
+        self.name,self.version,self.platform = details
 
     def lock(self,num_retries=0):
         """Lock the application directory for exclusive write access.
@@ -199,7 +209,7 @@ class Esky(object):
         #  Do I already own the lock?
         if os.path.exists(os.path.join(lockdir,myid)):
             #  Update file mtime to keep it safe from breakers
-            open(os.path.join(lockdir,myid),"wb").close()
+            os.utime(os.path.join(lockdir,myid),None)
             self._lock_count += 1
             return True
         #  Try to make the "locked" directory.
@@ -257,34 +267,46 @@ class Esky(object):
         appdir = self.appdir
         self.lock()
         try:
-            cur_version = get_best_version(appdir)
+            best_version = get_best_version(appdir)
             new_version = get_best_version(appdir,include_partial_installs=True)
             #  If there's a partial install we must complete it, since it
             #  could have left exes in the bootstrap env and we don't want
             #  to accidentally delete their dependencies.
-            if cur_version != new_version:
+            if best_version != new_version:
                 (_,v,_) = split_app_version(new_version)
                 self.install_version(v)
-                cur_version = new_version
+                best_version = new_version
             #  Now we can safely remove anything that's not part of the
-            #  current version's bootstrap env.
-            manifest = self._version_manifest(cur_version)
+            #  best version's bootstrap env.  Exceptions are the best
+            #  version itself, the currently-executing version, and any
+            #  locked versions.
+            manifest = self._version_manifest(best_version)
+            manifest.add("updates")
+            manifest.add("locked")
+            manifest.add(best_version)
+            if self.active_version:
+                manifest.add(self.active_version)
             for nm in os.listdir(appdir):
-                fullnm = os.path.join(appdir,nm)
-                if os.path.isdir(fullnm):
-                    if nm not in ("updates","locked",cur_version):
-                        if nm not in manifest:
-                            self._try_remove(fullnm)
-                else:
-                    if nm not in manifest:
-                        self._try_remove(fullnm)
+                if nm not in manifest:
+                    fullnm = os.path.join(appdir,nm)
+                    bsfile = os.path.join(fullnm,"esky-bootstrap.txt")
+                    if os.path.exists(bsfile):
+                        (_,v,_) = split_app_version(nm)
+                        try:
+                            self.uninstall_version(v)
+                        except VersionLockedError:
+                            pass
+                        else:
+                            self._try_remove(appdir,nm,manifest)
+                    else:
+                        self._try_remove(appdir,nm,manifest)
             if self.version_finder is not None:
                 self.version_finder.cleanup(self)
         finally:
             self.unlock()
 
-    def _try_remove(self,path):
-        """Try to remove the file/directory at the given path.
+    def _try_remove(self,appdir,path,manifest):
+        """Try to remove the file/directory at the given path in the appdir.
 
         This method attempts to remove the file or directory at the given path,
         but will fail silently under a number of conditions:
@@ -292,19 +314,24 @@ class Esky(object):
             * if a file is locked or permission is denied
             * if a directory cannot be emptied of all contents
             * if the path appears on sys.path
+            * if the path appears in the given manifest
 
         """
-        if path not in sys.path:
-            try:
-                if os.path.isdir(path):
-                    for nm in os.listdir(path):
-                        self._try_remove(os.path.join(path,nm))
-                    os.rmdir(path)
-                else:
-                    os.unlink(path)
-            except EnvironmentError, e:
-                if e.errno not in self._errors_to_ignore:
-                    raise
+        fullpath = os.path.join(appdir,path)
+        if fullpath in sys.path:
+            return
+        if path in manifest:
+            return
+        try:
+            if os.path.isdir(fullpath):
+                for nm in os.listdir(fullpath):
+                    self._try_remove(appdir,os.path.join(path,nm),manifest)
+                os.rmdir(fullpath)
+            else:
+                os.unlink(fullpath)
+        except EnvironmentError, e:
+            if e.errno not in self._errors_to_ignore:
+                raise
     _errors_to_ignore = (errno.ENOENT, errno.EPERM, errno.EACCES, errno.ENOTDIR,
                          errno.EISDIR, errno.EINVAL, errno.ENOTEMPTY,)
 
@@ -314,11 +341,14 @@ class Esky(object):
             raise NoVersionFinderError
         version = self.find_update()
         if version is not None:
-            if parse_version(version) > parse_version(self.version):
-                self.fetch_version(version)
-                self.install_version(version)
+            assert parse_version(version) > parse_version(self.version)
+            self.fetch_version(version)
+            self.install_version(version)
+            try:
                 self.uninstall_version(self.version)
-                self.reinitialize()
+            except VersionLockedError:
+                pass
+            self.reinitialize()
 
     def find_update(self):
         """Check for an available update to this app.
@@ -365,17 +395,13 @@ class Esky(object):
                 #  Move new bootrapping environment into main app dir.
                 #  Be sure to move dependencies before executables.
                 bootstrap = os.path.join(target,"esky-bootstrap")
-                if os.path.exists(os.path.join(bootstrap,"library.zip")):
-                    trn.move(os.path.join(bootstrap,"library.zip"),
-                             os.path.join(self.appdir,"library.zip"))
-                for nm in os.listdir(bootstrap):
-                    if is_core_dependency(nm):
-                        trn.move(os.path.join(bootstrap,nm),
-                                 os.path.join(self.appdir,nm))
-                for nm in os.listdir(bootstrap):
-                    if not is_core_dependency(nm) and nm != "library.zip":
-                        trn.move(os.path.join(bootstrap,nm),
-                                 os.path.join(self.appdir,nm))
+                with open(os.path.join(target,"esky-bootstrap.txt"),"rt") as f:
+                    for nm in f:
+                        nm = nm.strip()
+                        bssrc = os.path.join(bootstrap,nm)
+                        bsdst = os.path.join(self.appdir,nm)
+                        if os.path.exists(bssrc):
+                            trn.move(bssrc,bsdst)
                 #  Remove the bootstrap dir; the new version is now installed
                 trn.remove(bootstrap)
             except Exception:
@@ -394,27 +420,50 @@ class Esky(object):
         try:
             if not os.path.exists(target):
                 return
+            #  Clean up the bootstrapping environment in a transaction.
             trn = FSTransaction()
             try:
                 #  Get set of all files that must stay in the main appdir
                 to_keep = set()
                 for vname in os.listdir(self.appdir):
-                    if vname != target_name:
-                        to_keep.update(self._version_manifest(vname))
+                    if vname == target_name:
+                        continue
+                    details = split_app_version(vname)
+                    if details[0] != self.name:
+                        continue
+                    if parse_version(details[1]) < parse_version(version):
+                        continue
+                    to_keep.update(self._version_manifest(vname))
                 #  Remove files used only by the version being removed
                 to_rem = self._version_manifest(target_name) - to_keep
                 for nm in to_rem:
                     fullnm = os.path.join(self.appdir,nm)
                     trn.remove(fullnm)
-                #  Remove/disable the version.
-                #  To avoid trying to remove in-use files, we just clobber
-                #  the esky-bootstrap.txt file and leave the rest to cleanup().
-                trn.remove(os.path.join(target,"esky-bootstrap.txt"))
             except Exception:
                 trn.abort()
                 raise
             else:
                 trn.commit()
+            #  Disable the version by removing its esky-bootstrap.txt file.
+            #  To avoid clobbering in-use version, respect locks on this file.
+            bsfile = os.path.join(target,"esky-bootstrap.txt")
+            if sys.platform == "win32":
+                try:
+                    os.unlink(bsfile)
+                except EnvironmentError:
+                    raise VersionLockedError("version in use: %s" % (version,))
+            elif fcntl is not None:
+                f = open(bsfile,"r+")
+                try:
+                    fcntl.lockf(f.fileno(),fcntl.LOCK_EX|fcntl.LOCK_NB)
+                except OSError:
+                    raise VersionLockedError("version in use: %s" % (version,))
+                else:
+                    os.unlink(bsfile)
+                finally:
+                    f.close()
+            else:
+                os.unlink(bsfile)
         finally:
             self.unlock()
 
