@@ -180,6 +180,10 @@ def freeze(dist):
             pydll_bytes = f.read()
     else:
         pydll_bytes = None
+    #  Copy any core dependencies into the bootstrap env.
+    for nm in os.listdir(dist.freeze_dir):
+        if is_core_dependency(nm) and nm != pydll:
+            dist.copy_to_bootstrap_env(nm)
     #  Copy the loader program for each script into the bootstrap env.
     for exe in dist.get_executables():
         exepath = dist.copy_to_bootstrap_env(exe.name)
@@ -192,10 +196,6 @@ def freeze(dist):
         #  Inline the pythonXY.dll as a resource in the exe.
         if pydll_bytes is not None:
             winres.add_resource(exepath,pydll_bytes,pydll.upper(),1,0)
-    #  Copy any core dependencies into the bootstrap env.
-    for nm in os.listdir(dist.freeze_dir):
-        if is_core_dependency(nm) and nm != pydll:
-            dist.copy_to_bootstrap_env(nm)
 
 
 #  Code to fake out any bootstrappers that try to import from esky.
@@ -214,9 +214,10 @@ sys.modules["esky.bootstrap"] = __fake()
 #  We need to read the script to execute as a resource from the exe, so this
 #  only works if we can bootstrap a working ctypes module.  We then insert
 #  the source code from esky.winres directly into this function.
+#
 _CUSTOM_WIN32_CHAINLOADER = """
-_orig_chainload = chainload
-def chainload(target_dir):
+_orig_chainload = _chainload
+def _chainload(target_dir):
   # winres imports sys, making it a local variable.
   # Grab it here to avoid UnboundLocal errors
   import sys
