@@ -35,9 +35,14 @@ import errno
 if "posix" in sys.builtin_module_names:
     from posix import listdir, stat, unlink, rename, execv
     SEP = "/"
-elif "nt" in  sys.builtin_module_names:
+    try:
+        import fcntl
+    except ImportError:
+        fcntl = None
+elif "nt" in sys.builtin_module_names:
     from nt import listdir, stat, unlink, rename, spawnv, P_WAIT
     SEP = "\\"
+    fcntl = None
     def execv(filename,args):
         res = spawnv(P_WAIT,filename,args)
         raise SystemExit(res)
@@ -69,6 +74,25 @@ def exists(path):
     else:
         return True
 
+def appdir_from_executable(exepath):
+    """Find the top-level application directory, given sys.executable.
+
+    Ordinarily this would simply be the directory containing the executable,
+    but when running via a bundle on OSX the executable will be located at
+    <appdir>/Contents/MacOS/<exe>.
+    """
+    appdir = dirname(exepath)
+    if sys.platform == "darwin" and basename(appdir) == "MacOS":
+        # Looks like we might be in an app bundle.
+        appdir = dirname(appdir)
+        if basename(appdir) == "Contents":
+            # Yep, definitely in a bundle
+            appdir = dirname(appdir)
+        else:
+            # Nope, some other crazy scheme
+            appdir = dirname(exepath)
+    return appdir
+
 
 def bootstrap():
     """Bootstrap an esky frozen app into the newest available version.
@@ -77,7 +101,7 @@ def bootstrap():
     numbered version of the application that is fully installed, then
     chain-loads that version of the application.
     """
-    appdir = dirname(sys.executable)
+    appdir = appdir_from_executable(sys.executable)
     best_version = get_best_version(appdir)
     if best_version is None:
         raise RuntimeError("no usable frozen versions were found")
@@ -89,8 +113,40 @@ def chainload(target_dir):
 
     This function replaces the currently-running executable with the equivalent
     executable from the given target directory.
+
+    On platforms that support it, this also locks the target directory so that
+    it will not be removed by any simultaneously-running instances of the
+    application.
     """
-    target_exe = pathjoin(target_dir,basename(sys.executable))
+    global _version_dir_lockfile
+    lockfile = pathjoin(target_dir,"esky-bootstrap.txt")
+    try:
+        #  On windows, holding the file open is enough to lock it.
+        #  On other platforms, try for a shared lock using fcntl.
+        _version_dir_lockfile = open(lockfile,"r")
+        if "nt" not in sys.builtin_module_names:
+            if fcntl is not None:
+                fd = _version_dir_lockfile.fileno()
+                fcntl.lockf(fd,fcntl.LOCK_SH)
+    except EnvironmentError:
+        #  If the lockfile has gone missing, the version is being uninstalled.
+        #  Our only option is to re-execute ourself and find the new version.
+        if exists(lockfile):
+            raise
+        execv(sys.executable,sys.argv)
+    else:
+        #  If all goes well, we can actually launch the target version.
+        _chainload(target_dir)
+
+
+def _chainload(target_dir):
+    """Default implementation of the chainload() function.
+
+    Specific freezer modules may provide a more efficient, reliable or
+    otherwise better version of this function.
+    """
+    appdir = dirname(target_dir)
+    target_exe = target_dir + sys.executable[len(appdir):]
     execv(target_exe,[target_exe] + sys.argv[1:])
 
 
