@@ -10,6 +10,9 @@ import shutil
 import zipfile
 import threading
 import tempfile
+import urllib2
+import hashlib
+import tarfile
 import time
 from SimpleHTTPServer import SimpleHTTPRequestHandler
 from BaseHTTPServer import HTTPServer
@@ -18,18 +21,23 @@ from distutils.core import setup as dist_setup
 from distutils import dir_util
 
 import esky
+import esky.patch
 from esky import bdist_esky
 from esky.util import extract_zipfile, get_platform
 from esky.fstransact import FSTransaction
 
-try:
-    import bbfreeze
-except ImportError:
-    bbfreeze = None
 try:
     import py2exe
 except ImportError:
     py2exe = None
+try:
+    import py2app
+except ImportError:
+    py2app = None
+try:
+    import bbfreeze
+except ImportError:
+    bbfreeze = None
 try:
     import cx_Freeze
 except ImportError:
@@ -50,16 +58,21 @@ if not hasattr(HTTPServer,"shutdown"):
 
 class TestEsky(unittest.TestCase):
 
-  if bbfreeze is not None:
-    def test_esky_bbfreeze(self):
-        """Build and launch a self-testing esky application using bbfreeze."""
-        self._run_eskytester({"bdist_esky":{"freezer_module":"bbfreeze"}})
-
   if py2exe is not None:
     def test_esky_py2exe(self):
         """Build and launch a self-testing esky application using py2exe."""
         self._run_eskytester({"bdist_esky":{"freezer_module":"py2exe"}})
 
+  if py2app is not None:
+    def test_esky_py2app(self):
+        """Build and launch a self-testing esky application using py2app."""
+        self._run_eskytester({"bdist_esky":{"freezer_module":"py2app"}})
+
+  if bbfreeze is not None:
+    def test_esky_bbfreeze(self):
+        """Build and launch a self-testing esky application using bbfreeze."""
+        self._run_eskytester({"bdist_esky":{"freezer_module":"bbfreeze"}})
+
   if cx_Freeze is not None:
     def test_esky_cxfreeze(self):
         """Build and launch a self-testing esky application using cx_Freeze."""
@@ -88,18 +101,19 @@ class TestEsky(unittest.TestCase):
                 shutil.rmtree(build_dir)
         dir_util._path_created.clear()
         #  Build three increasing versions of the test package.
-        #  Version 0.2 will include a bundled MSVCRT on win32
+        #  Version 0.2 will include a bundled MSVCRT on win32.
+        #  Version 0.3 will be distributed as a patch.
         metadata = dict(name="eskytester",packages=["eskytester"],author="rfk",
                         description="the esky test package",
                         data_files=[("data",["eskytester/datafile.txt"])],
-                        package_data={"eskytester":["pkgdata.txt"]},
-                        script_args=["bdist_esky"])
+                        package_data={"eskytester":["pkgdata.txt"]},)
         options2 = options.copy()
         options2["bdist_esky"] = options["bdist_esky"].copy()
         options2["bdist_esky"]["bundle_msvcrt"] = True
-        dist_setup(version="0.1",scripts=["eskytester/script1.py"],options=options,**metadata)
-        dist_setup(version="0.2",scripts=["eskytester/script1.py","eskytester/script2.py"],options=options2,**metadata)
-        dist_setup(version="0.3",scripts=["eskytester/script2.py","eskytester/script3.py"],options=options,**metadata)
+        dist_setup(version="0.1",scripts=["eskytester/script1.py"],options=options,script_args=["bdist_esky"],**metadata)
+        dist_setup(version="0.2",scripts=["eskytester/script1.py","eskytester/script2.py"],options=options2,script_args=["bdist_esky"],**metadata)
+        dist_setup(version="0.3",scripts=["eskytester/script2.py","eskytester/script3.py"],options=options,script_args=["bdist_esky_patch"],**metadata)
+        os.unlink(os.path.join("dist","eskytester-0.3.%s.zip"%(platform,)))
         #  Serve the updates at http://localhost:8000/dist/
         print "running local update server"
         server = HTTPServer(("localhost",8000),SimpleHTTPRequestHandler)
@@ -112,9 +126,17 @@ class TestEsky(unittest.TestCase):
         extract_zipfile(zfname,deploydir)
         #  Run the first script, which will perform the necessary tests,
         #  launch script2 and script3, and write the file "tests-completed".
+        if options["bdist_esky"]["freezer_module"] == "py2app":
+            tests_completed = os.path.join(deploydir,"eskytester-0.3."+platform,"Contents/Resources/tests-completed")
+        else:
+            tests_completed = "tests-completed"
+        if os.path.exists(tests_completed):
+            os.unlink(tests_completed)
         if os.path.exists("tests-completed"):
             os.unlink("tests-completed")
-        if sys.platform == "win32":
+        if options["bdist_esky"]["freezer_module"] == "py2app":
+            cmd = os.path.join(deploydir,"Contents","MacOS","script1")
+        elif sys.platform == "win32":
             cmd = os.path.join(deploydir,"script1.exe")
         else:
             cmd = os.path.join(deploydir,"script1")
@@ -123,13 +145,12 @@ class TestEsky(unittest.TestCase):
         (stdout,_) = p.communicate()
         sys.stdout.write(stdout.decode())
         assert p.returncode == 0
-        assert os.path.exists("tests-completed")
-        os.unlink("tests-completed")
+        assert os.path.exists(tests_completed)
+        os.unlink(tests_completed)
     finally:
         os.chdir(olddir)
         if server:
             server.shutdown()
-
  
   def test_esky_locking(self):
     """Test that locking an Esky works correctly."""
@@ -424,3 +445,79 @@ class TestFSTransact(unittest.TestCase):
         self.assertContents("dir2","zero zero zero")
         self.assertContents("file0","zero zero zero")
 
+
+class TestPatch(unittest.TestCase):
+    """Testcases for esky.patch."""
+ 
+    _TEST_FILES = (
+        ("pyenchant-1.2.0.tar.gz","2fefef0868b110b1da7de89c08344dd2"),
+        ("pyenchant-1.5.2.tar.gz","fa1e4f3f3c473edd98c7bb0e46eea352"),
+        ("pyenchant-1.6.0.tar.gz","3fd7336989764d8d379a367236518439"),
+    )
+
+    _TEST_FILES_URL = "http://pypi.python.org/packages/source/p/pyenchant/"
+
+    def setUp(self):
+        self.tests_root = dirname(__file__)
+        platform = get_platform()
+        self.tfdir = tfdir = os.path.join(self.tests_root,"patch-test-files")
+        self.workdir = workdir = os.path.join(self.tests_root,"patch-test-temp."+platform)
+        if not os.path.isdir(tfdir):
+            os.makedirs(tfdir)
+        if not os.path.isdir(workdir):
+            os.makedirs(workdir)
+        #  Ensure we have the expected test files.
+        #  Download from PyPI if necessary.
+        for (tfname,hash) in self._TEST_FILES:
+            tfpath = os.path.join(tfdir,tfname)
+            if not os.path.exists(tfpath):
+                data = urllib2.urlopen(self._TEST_FILES_URL+tfname).read()
+                assert hashlib.md5(data).hexdigest() == hash
+                with open(tfpath,"wb") as f:
+                    f.write(data)
+
+    def tearDown(self):
+        shutil.rmtree(self.workdir)
+
+    def test_diffing_back_and_forth(self):
+        for (tf1,_) in self._TEST_FILES:
+            for (tf2,_) in self._TEST_FILES:
+                path1 = self._extract(tf1,"source")
+                path2 = self._extract(tf2,"target")
+                with open(os.path.join(self.workdir,"patch"),"wb") as f:
+                    esky.patch.write_patch(path1,path2,f)
+                if tf1 != tf2:
+                    self.assertNotEquals(esky.patch.calculate_digest(path1),
+                                         esky.patch.calculate_digest(path2))
+                with open(os.path.join(self.workdir,"patch"),"rb") as f:
+                    esky.patch.apply_patch(path1,f)
+                self.assertEquals(esky.patch.calculate_digest(path1),
+                                  esky.patch.calculate_digest(path2))
+
+    def test_apply_patch(self):
+        path1 = self._extract("pyenchant-1.2.0.tar.gz","source")
+        path2 = self._extract("pyenchant-1.6.0.tar.gz","target")
+        path1 = os.path.join(path1,"pyenchant-1.2.0")
+        path2 = os.path.join(path2,"pyenchant-1.6.0")
+        pf = os.path.join(self.tfdir,"v1.2.0_to_v1.6.0.patch")
+        if not os.path.exists(pf):
+            pf = os.path.join(self.tfdir,"../../../../../esky/tests/patch-test-files/v1.2.0_to_v1.6.0.patch")
+        with open(pf,"rb") as f:
+            esky.patch.apply_patch(path1,f)
+        self.assertEquals(esky.patch.calculate_digest(path1),
+                         esky.patch.calculate_digest(path2))
+        
+
+    def _extract(self,filename,dest):
+        dest = os.path.join(self.workdir,dest)
+        if os.path.exists(dest):
+            shutil.rmtree(dest)
+        f = tarfile.open(os.path.join(self.tfdir,filename),"r:gz")
+        try:
+            f.extractall(dest)
+        finally:
+            f.close()
+        return dest
+        
+
+
